Adding graceful termination feature, to help look for memory leaks.
[privoxy.git] / cgi.c
diff --git a/cgi.c b/cgi.c
index 641af64..98d05cd 100644 (file)
--- a/cgi.c
+++ b/cgi.c
@@ -1,4 +1,4 @@
-const char cgi_rcs[] = "$Id: cgi.c,v 1.44 2002/03/05 22:43:45 david__schmidt Exp $";
+const char cgi_rcs[] = "$Id: cgi.c,v 1.49 2002/03/13 00:27:04 jongfoster Exp $";
 /*********************************************************************
  *
  * File        :  $Source: /cvsroot/ijbswa/current/cgi.c,v $
@@ -38,6 +38,24 @@ const char cgi_rcs[] = "$Id: cgi.c,v 1.44 2002/03/05 22:43:45 david__schmidt Exp
  *
  * Revisions   :
  *    $Log: cgi.c,v $
+ *    Revision 1.49  2002/03/13 00:27:04  jongfoster
+ *    Killing warnings
+ *
+ *    Revision 1.48  2002/03/08 17:47:07  jongfoster
+ *    Adding comments
+ *
+ *    Revision 1.47  2002/03/08 16:41:33  oes
+ *    Added GIF images again
+ *
+ *    Revision 1.46  2002/03/07 03:48:38  oes
+ *     - Changed built-in images from GIF to PNG
+ *       (with regard to Unisys patent issue)
+ *     - Added a 4x4 pattern PNG which is less intrusive
+ *       than the logo but also clearly marks the deleted banners
+ *
+ *    Revision 1.45  2002/03/06 22:54:35  jongfoster
+ *    Automated function-comment nitpicking.
+ *
  *    Revision 1.44  2002/03/05 22:43:45  david__schmidt
  *    - Better error reporting on OS/2
  *    - Fix double-slash comment (oops)
@@ -315,6 +333,11 @@ static const struct cgi_dispatcher cgi_dispatchers[] = {
    { "",
          cgi_default,
          "Junkbuster main page" },
+#ifdef FEATURE_GRACEFUL_TERMINATION
+   { "die", 
+         cgi_die,  
+         "<b>Shut down</b> - <font color=red size='+1'>Do not deploy this build in a production environment, this is a one click Denial Of Service attack!!!</font>" }, 
+#endif
    { "show-status", 
          cgi_show_status,  
          "Show information about the current configuration" }, 
@@ -399,10 +422,10 @@ static const struct cgi_dispatcher cgi_dispatchers[] = {
          NULL /* Sends a robots.txt file to tell robots to go away. */ }, 
    { "send-banner",
          cgi_send_banner, 
-         NULL /* Send the transparent or \"Junkbuster\" gif */ },
+         NULL /* Send a built-in image */ },
    { "t",
-         cgi_transparent_gif
-         NULL /* Send a transparent gif (short name) */ },
+         cgi_transparent_image
+         NULL /* Send a transparent image (short name) */ },
    { NULL, /* NULL Indicates end of list and default page */
          cgi_error_404,
          NULL /* Unknown CGI page */ }
@@ -410,13 +433,62 @@ static const struct cgi_dispatcher cgi_dispatchers[] = {
 
 
 /*
- * Some images
+ * Bulit-in images for ad replacement
  *
- * Hint: You can encode your own GIFs like this:
- * perl -e 'while (read STDIN, $c, 1) { printf("\\%.3o,", unpack("C", $c)); }'
+ * Hint: You can encode your own images like this:
+ * cat your-image | perl -e 'while (read STDIN, $c, 1) { printf("\\%.3o", unpack("C", $c)); }'
+ */
+
+#ifdef FEATURE_NO_GIFS
+
+/*
+ * "Junkbuster" logo, as a PNG.
+ */
+const char image_logo_data[] =
+   "\211\120\116\107\015\012\032\012\000\000\000\015\111\110\104"
+   "\122\000\000\000\104\000\000\000\013\001\003\000\000\000\374"
+   "\323\271\110\000\000\000\004\147\101\115\101\000\000\261\217"
+   "\013\374\141\005\000\000\000\006\120\114\124\105\310\310\310"
+   "\000\000\000\062\100\100\345\000\000\000\001\164\122\116\123"
+   "\000\100\346\330\146\000\000\000\001\142\113\107\104\000\210"
+   "\005\035\110\000\000\000\126\111\104\101\124\170\332\143\140"
+   "\200\003\066\006\146\066\006\006\011\124\326\331\315\146\147"
+   "\316\333\174\116\000\262\156\247\347\334\234\071\373\001\220"
+   "\065\373\130\316\035\311\331\015\020\326\161\311\337\014\014"
+   "\151\147\147\247\345\034\236\331\014\144\335\235\155\236\173"
+   "\163\366\154\006\006\233\334\331\162\307\316\363\174\106\330"
+   "\300\000\000\002\127\040\324\250\204\044\102\000\000\000\000"
+   "\111\105\116\104\256\102\140\202";
+
+/*
+ * Checkerboard pattern, as a PNG.
  */
+const char image_pattern_data[] =
+   "\211\120\116\107\015\012\032\012\000\000\000\015\111\110\104"
+   "\122\000\000\000\004\000\000\000\004\010\002\000\000\000\046"
+   "\223\011\051\000\000\000\006\142\113\107\104\000\310\000\310"
+   "\000\310\052\045\225\037\000\000\000\032\111\104\101\124\170"
+   "\332\143\070\161\342\304\377\377\377\041\044\003\234\165\342"
+   "\304\011\006\234\062\000\125\200\052\251\125\174\360\223\000"
+   "\000\000\000\111\105\116\104\256\102\140\202";
 
-const char image_junkbuster_gif_data[] =
+/*
+ * 1x1 transparant PNG.
+ */
+const char image_blank_data[] =
+   "\211\120\116\107\015\012\032\012\000\000\000\015\111\110\104"
+   "\122\000\000\000\002\000\000\000\002\010\002\000\000\000\375"
+   "\324\232\163\000\000\000\006\142\113\107\104\000\310\000\310"
+   "\000\310\052\045\225\037\000\000\000\027\111\104\101\124\170"
+   "\332\143\374\377\377\377\311\223\047\031\116\234\070\361\377"
+   "\377\177\000\112\335\012\257\162\103\306\335\000\000\000\000"
+   "\111\105\116\104\256\102\140\202o";
+#else
+
+/*
+ * "Junkbuster" logo, as a GIF.
+ */
+const char image_logo_data[] =
    "GIF89aD\000\013\000\360\000\000\000\000\000\377\377\377!"
    "\371\004\001\000\000\001\000,\000\000\000\000D\000\013\000"
    "\000\002a\214\217\251\313\355\277\000\200G&K\025\316hC\037"
@@ -426,20 +498,32 @@ const char image_junkbuster_gif_data[] =
    "\372\003S\275\274k2\354\254z\347?\335\274x\306^9\374\276"
    "\037Q\000\000;";
 
-const int image_junkbuster_gif_length = sizeof(image_junkbuster_gif_data) - 1;
-
+/*
+ * Checkerboard pattern, as a GIF.
+ */
+const char image_pattern_data[] =
+   "\107\111\106\070\071\141\004\000\004\000\200\000\000\310\310"
+   "\310\377\377\377\041\376\025\111\040\165\163\145\144\040\164"
+   "\157\040\142\145\040\141\040\142\141\156\156\145\162\000\054"
+   "\000\000\000\000\004\000\004\000\000\002\005\104\174\147\270"
+   "\005\000\073";
 
-const char image_blank_gif_data[] =
+/*
+ * 1x1 transparant GIF.
+ */
+const char image_blank_data[] =
    "GIF89a\001\000\001\000\200\000\000\377\377\377\000\000"
    "\000!\371\004\001\000\000\000\000,\000\000\000\000\001"
    "\000\001\000\000\002\002D\001\000;";
+#endif
 
-const int image_blank_gif_length = sizeof(image_blank_gif_data) - 1;
+const size_t image_logo_length    = sizeof(image_logo_data) - 1;
+const size_t image_pattern_length = sizeof(image_pattern_data) - 1;
+const size_t image_blank_length   = sizeof(image_blank_data) - 1;
 
 
 static struct http_response cgi_error_memory_response[1];
 
-
 static struct http_response *dispatch_known_cgi(struct client_state * csp,
                                                 const char * path);
 static struct map *parse_cgi_parameters(char *argstring);
@@ -587,7 +671,8 @@ static struct http_response *dispatch_known_cgi(struct client_state * csp,
                             csp->ip_addr_str, csp->http->cmd); 
 
    /* Find and start the right CGI function*/
-   for (d = cgi_dispatchers; FOREVER; d++)
+   d = cgi_dispatchers;
+   for (;;)
    {
       if ((d->name == NULL) || (strcmp(path_copy, d->name) == 0))
       {
@@ -610,6 +695,7 @@ static struct http_response *dispatch_known_cgi(struct client_state * csp,
             return cgi_error_memory();
          }
       }
+      d++;
    }
 }